/**
 * @fileOverview  Auxiliary data management procedures
 * @author Gerd Wagner and Juan-Francisco Reyes
 */
import Author from "../m/Author.mjs";
import Publisher from "../m/Publisher.mjs";
import Book from "../m/Book.mjs";
import { db } from "../c/initialize.mjs";

/*******************************************
 *** Auxiliary methods for testing **********
 ********************************************/
/**
 *  Load and save test data
 */
async function generateTestData () {
  try {
    let response = null;
    // generate author records
    console.log("Generating test data...");
    response = await fetch( "../../test-data/authors.json");
    const authorRecords = await response.json();
    await Promise.all( authorRecords.map(
      authorRec => Author.add( authorRec)
    ));
    console.log(`${authorRecords.length} author records generated`);
    // generate publisher records
    response = await fetch( "../../test-data/publishers.json");
    const publisherRecords = await response.json();
    await Promise.all( publisherRecords.map(
      publisherRec => Publisher.add( publisherRec)
    ));
    console.log(`${publisherRecords.length} publisher records generated`);
    // generate books records
    response = await fetch( "../../test-data/books.json");
    const bookRecords = await response.json();
    await Promise.all( bookRecords.map(
      bookRec => Book.add( bookRec)
    ));
    console.log( `${bookRecords.length} book records generated`);
  } catch (e) {
    console.error(`${e.constructor.name} : ${e.message}`);
  }
}
/**
 * Clear data
 */
async function clearData () {
  try {
    if (confirm("Do you really want to delete all test data?")) {
      // retrieve all records
      console.log("Cleaning test data...");
      const authorDocSns = (await db.collection("authors").withConverter( Author.converter)
        .get()).docs;
      for (const a of authorDocSns) {
        await db.collection("authors").doc(a.id).delete();
        console.log(`Author record ${a.id} deleted!`);
      }
      console.log(`${authorDocSns.length} author records deleted`);
      const publisherDocSns = (await db.collection("publishers").withConverter( Publisher.converter)
        .get()).docs;
      await Promise.all( publisherDocSns.map(
        publisherDocSn => Publisher.destroy( publisherDocSn.id)
      ));
      console.log(`${publisherDocSns.length} publisher records deleted`);
      const bookDocSns = (await db.collection("books").withConverter( Book.converter)
        .get()).docs;
      await Promise.all( bookDocSns.map(
        bookDocSn => Book.destroy( bookDocSn.id)
      ));
      console.log(`${bookDocSns.length} book records deleted`);
    }
  } catch (e) {
    console.error(`${e.constructor.name} : ${e.message}`);
  }
}

export { generateTestData, clearData };
